from typing import Dict, Any, List
from .internal import meets_crit

def any_meet_criteria(records: List[Dict[str, Any]], criteria: Dict[str, Any]) -> bool:
    """
    Determine if any of the records meet the given criteria.

    Parameters:
    - records (List[Dict[str, Any]]): The data records.
    - criteria (Dict[str, Any]): The criteria. Should be a dictionary representing the conditions that the record must satisfy. May include nested conditions.

    Returns:
    - bool: Whether any of the records meet the criteria.
    """
    return any(meets_crit(record, criteria) for record in records)



from json_criteria import meets_criteria

record = { 'name': 'Joe', 'age': 30, 'department': 'A13' }

# age < 40 AND (department starts with 'B' OR department ends with '13')
criteria = {
    'AND': [
        { 'key': 'age', 'op': 'less_than', 'value': 40 },
        { 'OR': [
            { 'key': 'department', 'op': 'starts_with', 'value': 'B' },
            { 'key': 'department', 'op': 'ends_with', 'value': '13' }
        ]},
    ]
}

# `result` will be True
result = meets_criteria(record, criteria)



import unittest

from src.json_criteria.internal import compare

class TestCompare(unittest.TestCase):
    def test_equal_to_true(self):
        result = compare(1, 'equal_to', 1)
        self.assertTrue(result)

    def test_equal_to_false(self):
        result = compare(1, 'equal_to', 2)
        self.assertFalse(result)

    def test_not_equal_to_true(self):
        result = compare(1, 'not_equal_to', 2)
        self.assertTrue(result)

    def test_not_equal_to_false(self):
        result = compare(1, 'not_equal_to', 1)
        self.assertFalse(result)

    def test_in_true(self):
        result = compare('Test', 'in', ['Test'])
        self.assertTrue(result)

    def test_in_false(self):
        result = compare('Test', 'in', ['Test2'])
        self.assertFalse(result)

    def test_not_in_true(self):
        result = compare('Test', 'not_in', ['abc'])
        self.assertTrue(result)

    def test_not_in_false(self):
        result = compare('Test', 'not_in', ['Test'])
        self.assertFalse(result)

    def test_contains_true(self):
        result = compare('[Test]', 'contains', 'Test')
        self.assertTrue(result)

    def test_contains_false(self):
        result = compare('[Test]', 'contains', 'Test2')
        self.assertFalse(result)

    def test_not_contains_true(self):
        result = compare('[Test]', 'not_contains', 'Hello')
        self.assertTrue(result)

    def test_not_contains_false(self):
        result = compare('[Test]', 'not_contains', 'Test')
        self.assertFalse(result)

    def test_matches_regex_true(self):
        result = compare('Hello World', 'matches_regex', '^[a-zA-Z]*\\s[a-zA-Z]*$')
        self.assertTrue(result)

    def test_matches_regex_false(self):
        result = compare('Hello World', 'matches_regex', '^[A-Z]*\\s[A-Z]*$')
        self.assertFalse(result)

    def test_unsupported_operator(self):
        with self.assertRaises(ValueError) as context:
            compare(1, 'unsupported', 1)
        
        self.assertEqual(str(context.exception), 'Unsupported operator: unsupported')

if __name__ == '__main__':
    unittest.main()



from typing import Dict, Any, List, Callable
from .internal import meets_crit

def apply_using_criteria(
    records: List[Dict[str, Any]],
    criteria: Dict[str, Any],
    func: Callable[[Dict[str, Any]], Dict[str, Any]]
) -> List[Dict[str, Any]]:
    """
    Apply a specified function to each record that meets the given criteria.

    Parameters:
    - records (List[Dict[str, Any]]): The data records.
    - criteria (Dict[str, Any]): The criteria. Should be a dictionary representing the conditions that the record must satisfy. May include nested conditions.
    - func (Callable): Function to apply to records that meet the criteria.

    Returns:
    - list: List of records with the specified function applied.
    """
    result_records = []
    for record in records:
        if meets_crit(record, criteria):
            result_records.append(func(record))
        else:
            result_records.append(record)
    return result_records



from typing import Dict, Any
from .compare import compare

def meets_crit(record: Dict[str, Any], element: Dict[str, Any]) -> bool:
    """
    Internal function to check if a record meets the given criteria.

    Parameters:
    - record (Dict[str, Any]): The data record.
    - element (Dict[str, Any]): The element of the criteria.

    Returns:
    - bool: Whether or not the record meets the criteria.
    """

    if not isinstance(element, dict):
        raise ValueError('Invalid element: must be a dictionary')

    # AND, OR, NOT operators
    if 'AND' in element:
        return all(meets_crit(record, el) for el in element.get('AND', []))
    if 'OR' in element:
        return any(meets_crit(record, el) for el in element.get('OR', []))
    if 'NOT' in element:
        return not meets_crit(record, element.get('NOT', {}))

    # Conditions
    key = element.get('key')
    operator = element.get('op')
    value = element.get('value')

    if not isinstance(key, str) or not operator or value is None:
        raise ValueError(f'Invalid condition: {element}')

    return compare(record.get(key), operator, value)



import unittest

from src.json_criteria import all_meet_criteria

class TestAllMeetCriteria(unittest.TestCase):
    def test_true(self):
        records = [{ 'name': 'Joe' , 'age': 30 }, { 'name': 'Steve', 'age': 60 }]
        criteria = { 'key': 'age', 'op': 'greater_than_or_equal_to', 'value': 30 }
        result = all_meet_criteria(records, criteria)
        self.assertTrue(result)

    def test_false(self):
        records = [{ 'name': 'Joe' , 'age': 30 }, { 'name': 'Steve', 'age': 60 }]
        criteria = { 'key': 'age', 'op': 'equal_to', 'value': 30 }
        result = all_meet_criteria(records, criteria)
        self.assertFalse(result)

if __name__ == '__main__':
    unittest.main()



import unittest

from src.json_criteria import meets_criteria

class TestMeetsCriteria(unittest.TestCase):
    def test_basic(self):
        record = { 'name': 'Joe' , 'age': 30 }
        criteria = { 'key': 'age', 'op': 'equal_to', 'value': 30 }
        result = meets_criteria(record, criteria)
        self.assertTrue(result)

    def test_basic2(self):
        record = { 'name': 'Joe' , 'age': 30 }
        criteria = { 'AND': [
            {'key': 'age', 'op': 'equal_to', 'value': 30 },
            {'key': 'name', 'op': 'equal_to', 'value': 'Joe' }
            ]
        }
        result = meets_criteria(record, criteria)
        self.assertTrue(result)

    def test_basic3(self):
        record = { 'name': 'Joe' , 'age': 30 }
        criteria = { 'AND': [
            {'key': 'age', 'op': 'equal_to', 'value': 30 },
            {'key': 'name', 'op': 'equal_to', 'value': 'Steve' }
            ]
        }
        result = meets_criteria(record, criteria)
        self.assertFalse(result)

if __name__ == '__main__':
    unittest.main()



from typing import Dict, Any
from .internal import meets_crit

def meets_criteria(record: Dict[str, Any], criteria: Dict[str, Any]) -> bool:
    """
    Determine if a record meets the given criteria.

    Parameters:
    - record (Dict[str, Any]): The data record.
    - criteria (Dict[str, Any]): The criteria. Should be a dictionary representing the conditions that the record must satisfy. May include nested conditions.

    Returns:
    - bool: Whether or not the record meets the criteria.
    """
    return meets_crit(record, criteria)
